#!/bin/python

import tensorflow as tf
import numpy as np
import matplotlib
import matplotlib.pyplot as plt

#font = {'family' : 'Adobe Caslon Pro',
#        'size'   : 10}

#matplotlib.rc('font', **font)

def model_variable(shape, name):
        variable = tf.get_variable(name=name,
                                   dtype=tf.float32,
                                   shape=shape,
                                   initializer=tf.random_normal_initializer(stddev=0.1)
        )
        tf.add_to_collection('model_variables', variable)
        tf.add_to_collection('l2', tf.reduce_sum(tf.pow(variable,2)))
        return variable
    
class Model():
    def __init__(self, sess, data, nEpochs, learning_rate, lambduh):
        self.sess = sess
        self.data = data
        self.nEpochs = nEpochs
        self.learning_rate = learning_rate
        self.lambduh = lambduh
        self.build_model()
        
    def build_model(self):
        self.x = tf.placeholder(tf.float32, shape=[])
        self.y = tf.placeholder(tf.float32, shape=[])
        
        w = model_variable([], 'w')
        b = model_variable([], 'b')
            
        self.yhat =  w*self.x + b

        self.mse = tf.reduce_mean(tf.pow(self.yhat - self.y, 2))
        self.l2_penalty = tf.reduce_sum(tf.get_collection('l2'))
        self.loss = self.mse + self.lambduh*self.l2_penalty
        
    def train_init(self):
        model_variables = tf.get_collection('model_variables')            
        self.optim = (
            tf.train.GradientDescentOptimizer(learning_rate=0.1)
            .minimize(self.loss, var_list=model_variables)
            )
        self.sess.run(tf.initialize_all_variables())

    def train_iter(self, x, y):
        loss, mse, l2_penalty, _ = self.sess.run([self.loss, self.mse, self.l2_penalty, self.optim],
                                          feed_dict={self.x : x, self.y : y})
        print('loss: {}, mse: {}, l2_penalty {}'.format(loss, mse, l2_penalty))

    def train(self):
        for _ in range(self.nEpochs):
            for x, y in self.data():
                self.train_iter(x, y)

    def infer(self, x):
        return self.sess.run(self.yhat, feed_dict={self.x : x})

def data():
    num_samp = 50
    sigma = 0.5
    w = 5
    b = 2
    np.random.seed(31415)
    for _ in range(num_samp):
        x = np.random.uniform()
        y = w*x + b + np.random.normal()*sigma
        yield x, y

sess = tf.Session()
model = Model(sess, data, nEpochs=10, learning_rate=1e-2, lambduh=1e-4)
model.train_init()
model.train()
    
print(sess.run(tf.get_collection('model_variables')))

# generate manifold and plot

x= np.linspace(-0.1, 1.1, 100)

y = []
for a in x:
    y.append(model.infer(a))
y = np.array(y)

examples, targets = zip(*list(data()))

fig, ax = plt.subplots(1,1)
fig.set_size_inches(5, 3)
plt.plot(x, y, '-', np.array(examples), np.array(targets), 'o')
plt.xlim([-0.1, 1.1])
plt.ylim([0, 7.5])
ax.set_xlabel('x')
ax.set_ylabel('y').set_rotation(0)
plt.title('Linear Regression')
plt.tight_layout()
plt.savefig('plot.pdf', format='pdf', bbox_inches='tight')
